@@ -37,8 +37,27 @@ class sRNATargetPrediction(object):
             if gff.endswith(".gff"):
                 self.helper.check_uni_attributes(os.path.join(gffs, gff))
 
+    def _check_long_id(self, seq_file, long_ids, type_):
+        out_file = seq_file + "_tmp.fa"
+        out = open(out_file, "w")
+        with open(seq_file) as f_h:
+            for line in f_h:
+                line = line.strip()
+                if line.startswith(">"):
+                    if len(line) > 40:
+                        long_ids[type_].append(line[1:])
+                        out.write(">TMP" + type_ + "_" + str(len(long_ids[type_])) + "\n")
+                    else:
+                        out.write(line + "\n")
+                else:
+                    out.write(line + "\n")
+        out.close()
+        return out_file
+
+
+
     def _run_rnaplfold(self, rnaplfold_path, file_type, win_size, span,
-                       unstr_region, seq_path, prefix, out_path, log):
+                       unstr_region, long_ids, seq_path, prefix, out_path, log):
         current = os.getcwd()
         os.chdir(out_path)
         command = " ".join([rnaplfold_path,
@@ -47,17 +66,21 @@ class sRNATargetPrediction(object):
                             "-u", str(unstr_region),
                             "-O"])
         if file_type == "sRNA":
-            log.write("<".join([command, os.path.join(current, seq_path,
+            srna_seq_file = os.path.join(current, seq_path,
                                 "_".join([self.tmps["tmp"], prefix,
-                                          file_type + ".fa"]))]) + "\n")
-            os.system("<".join([command, os.path.join(current, seq_path,
-                                "_".join([self.tmps["tmp"], prefix,
-                                          file_type + ".fa"]))]))
+                                          file_type + ".fa"]))
+            out_file = self._check_long_id(srna_seq_file, long_ids, "srna")
+            log.write("<".join([command, out_file]) + "\n")
+            os.system("<".join([command, out_file]))
         else:
-            log.write("<".join([command, os.path.join(current, seq_path,
-                                "_".join([prefix, file_type + ".fa"]))]) + "\n")
-            os.system("<".join([command, os.path.join(current, seq_path,
-                                "_".join([prefix, file_type + ".fa"]))]))
+            tar_seq_file = os.path.join(current, seq_path,
+                                "_".join([prefix, file_type + ".fa"]))
+            for tar_seq_file in os.listdir(os.path.join(current, seq_path)):
+                if (prefix + "_" + file_type + "_") in tar_seq_file:
+                    out_file = self._check_long_id(os.path.join(
+                        current, seq_path, tar_seq_file), long_ids, "tar")
+                    log.write("<".join([command, out_file]) + "\n")
+                    os.system("<".join([command, out_file]))
         os.chdir(current)
 
     def _wait_process(self, processes):
@@ -135,7 +158,48 @@ class sRNATargetPrediction(object):
             gff_f.close()
             out.close()
 
-    def _gen_seq(self, prefixs, args_tar):
+    def _gen_seq(self, prefixs, target_prefixs, args_tar):
+        print("Generating sRNA fasta files")
+        for gff in os.listdir(self.gff_path):
+            if gff.endswith(".gff"):
+                prefix = gff.replace(".gff", "")
+                target_prefixs.append(prefix)
+        detect = False
+        for gff in os.listdir(self.gff_path):
+            if gff.endswith(".gff"):
+                prefix = gff.replace(".gff", "")
+                potential_target(os.path.join(self.gff_path, gff),
+                                 os.path.join(self.fasta_path, prefix + ".fa"),
+                                 os.path.join(self.target_seq_path), args_tar, target_prefixs)
+                file_num = 1
+                num = 0
+                sub_prefix = os.path.join(self.target_seq_path,
+                                          "_".join([prefix, "target"]))
+                if os.path.exists(sub_prefix + ".fa"):
+                    sub_out = open("_".join([sub_prefix, str(file_num) + ".fa"]),
+                                   "w")
+                    with open((sub_prefix + ".fa"), "r") as t_f:
+                        for line in t_f:
+                            line = line.strip()
+                            if line.startswith(">"):
+#                                line = line.replace("|", "_")
+                                num += 1
+                            if (num == 100):
+                                num = 0
+                                file_num += 1
+                                sub_out.close()
+                                sub_out = open("_".join([sub_prefix,
+                                               str(file_num) + ".fa"]), "w")
+                            detect = True
+                            sub_out.write(line + "\n")
+                    sub_out.close()
+                else:
+                    open(sub_prefix + ".fa", "w").close()
+        if not detect:
+            print("No assigned features can be found. "
+              "Please check your genome annotation. "
+              "And assign correct features to --target_feature.")
+            sys.exit()
         print("Generating sRNA fasta files")
         for srna in os.listdir(self.srna_path):
             if srna.endswith("_sRNA.gff"):
@@ -157,41 +221,15 @@ class sRNATargetPrediction(object):
                             os.path.join(self.fasta_path, prefix + ".fa"),
                             srna_out, args_tar.query)
                 self._sort_srna_fasta(srna_out, prefix, self.srna_seq_path)
-        print("Generating target fasta files")
-        for gff in os.listdir(self.gff_path):
-            if gff.endswith(".gff"):
-                prefix = gff.replace(".gff", "")
-                potential_target(os.path.join(self.gff_path, gff),
-                                 os.path.join(self.fasta_path, prefix + ".fa"),
-                                 os.path.join(self.target_seq_path), args_tar)
-                file_num = 1
-                num = 0
-                sub_prefix = os.path.join(self.target_seq_path,
-                                          "_".join([prefix, "target"]))
-                sub_out = open("_".join([sub_prefix, str(file_num) + ".fa"]),
-                               "w")
-                with open((sub_prefix + ".fa"), "r") as t_f:
-                    for line in t_f:
-                        line = line.strip()
-                        if line.startswith(">"):
-#                            line = line.replace("|", "_")
-                            num += 1
-                        if (num == 100):
-                            num = 0
-                            file_num += 1
-                            sub_out.close()
-                            sub_out = open("_".join([sub_prefix,
-                                           str(file_num) + ".fa"]), "w")
-                        sub_out.write(line + "\n")
-                sub_out.close()
+
 
     def _run_rnaplex(self, prefix, rnaplfold_folder, args_tar, log):
         print("Running RNAplex of {0}".format(prefix))
         num_process = 0
         processes = []
         for seq in os.listdir(self.target_seq_path):
-            if (prefix in seq) and ("_target_" in seq):
-                print("Running RNAplex with {0}".format(seq))
+            if ("_target_" in seq) and (".fa_tmp.fa" in seq):
+                print("Running RNAplex with {0}".format(seq.replace(".fa_tmp.fa", "")))
                 out_rnaplex = open(os.path.join(
                     self.rnaplex_path, prefix, "_".join([
                         prefix, "RNAplex", str(num_process) + ".txt"])), "w")
@@ -199,7 +237,7 @@ class sRNATargetPrediction(object):
                 log.write(" ".join([args_tar.rnaplex_path,
                            "-q", os.path.join(
                                self.srna_seq_path, "_".join([
-                                   self.tmps["tmp"], prefix, "sRNA.fa"])),
+                                   self.tmps["tmp"], prefix, "sRNA.fa_tmp.fa"])),
                            "-t", os.path.join(self.target_seq_path, seq),
                            "-l", str(args_tar.inter_length),
                            "-e", str(args_tar.energy),
@@ -208,7 +246,7 @@ class sRNATargetPrediction(object):
                 p = Popen([args_tar.rnaplex_path,
                            "-q", os.path.join(
                                self.srna_seq_path, "_".join([
-                                   self.tmps["tmp"], prefix, "sRNA.fa"])),
+                                   self.tmps["tmp"], prefix, "sRNA.fa_tmp.fa"])),
                            "-t", os.path.join(self.target_seq_path, seq),
                            "-l", str(args_tar.inter_length),
                            "-e", str(args_tar.energy),
@@ -225,25 +263,50 @@ class sRNATargetPrediction(object):
             log.write("\t" + os.path.join(self.rnaplex_path, prefix, file_) + "\n")
         return num_process
 
-    def _rna_plex(self, prefixs, args_tar, log):
+    def _restore_long_ids(self, rnaplex_file, long_ids):
+        out = open(rnaplex_file + "tmp", "w")
+        with open(rnaplex_file, "r") as t_f:
+            for line in t_f:
+                line = line.strip()
+                if (line.startswith(">")):
+                    if (line.startswith(">TMPtar_")):
+                        header = long_ids["tar"][int(line.split("_")[1]) - 1]
+                    elif (line.startswith(">TMPsrna_")):
+                        header = long_ids["srna"][int(line.split("_")[1]) - 1]
+                    else:
+                        header = line[1:]
+                    out.write(">" + header + "\n")
+                else:
+                    out.write(line + "\n")
+        out.close()
+        shutil.move(rnaplex_file + "tmp", rnaplex_file)
+
+    def _rna_plex(self, prefixs, target_prefixs, args_tar, log):
         log.write("Using RNAplex and RNAplfold to predict sRNA targets.\n")
         log.write("Please make sure the version of Vienna RNA package is "
                   "at least 2.3.2.\n")
+        tmp_rnaplfold_folder = os.path.join(self.rnaplex_path,
+                                          "tmp_RNAplfold")
+        if os.path.exists(tmp_rnaplfold_folder):
+            shutil.rmtree(tmp_rnaplfold_folder)
+        os.mkdir(tmp_rnaplfold_folder)
+        long_ids = {"tar": [], "srna": []}
+        for prefix in target_prefixs:
+            self._run_rnaplfold(
+                args_tar.rnaplfold_path, "target", args_tar.win_size_t,
+                args_tar.span_t, args_tar.unstr_region_rnaplex_t, long_ids,
+                self.target_seq_path, prefix, tmp_rnaplfold_folder, log)
         for prefix in prefixs:
             print("Running RNAplfold of {0}".format(prefix))
             self.helper.check_make_folder(
                         os.path.join(self.rnaplex_path, prefix))
             rnaplfold_folder = os.path.join(self.rnaplex_path, prefix,
                                           "RNAplfold")
-            os.mkdir(rnaplfold_folder)
+            shutil.copytree(tmp_rnaplfold_folder, rnaplfold_folder)
             self._run_rnaplfold(
                 args_tar.rnaplfold_path, "sRNA", args_tar.win_size_s,
-                args_tar.span_s, args_tar.unstr_region_rnaplex_s,
+                args_tar.span_s, args_tar.unstr_region_rnaplex_s, long_ids,
                 self.srna_seq_path, prefix, rnaplfold_folder, log)
-            self._run_rnaplfold(
-                args_tar.rnaplfold_path, "target", args_tar.win_size_t,
-                args_tar.span_t, args_tar.unstr_region_rnaplex_t,
-                self.target_seq_path, prefix, rnaplfold_folder, log)
             num_process = self._run_rnaplex(prefix, rnaplfold_folder, args_tar, log)
             rnaplex_file = os.path.join(self.rnaplex_path, prefix,
                                         "_".join([prefix, "RNAplex.txt"]))
@@ -256,6 +319,8 @@ class sRNATargetPrediction(object):
                     self.rnaplex_path, prefix, "_".join([
                         prefix, "RNAplex", str(index) + ".txt"])),
                     rnaplex_file)
+            if (len(long_ids["tar"]) != 0) or (len(long_ids["srna"]) != 0):
+                self._restore_long_ids(rnaplex_file, long_ids)
             log.write("\t" + rnaplex_file + " is generated.\n")
             self.helper.remove_all_content(os.path.join(
                  self.rnaplex_path, prefix), "_RNAplex_", "file")
@@ -331,7 +396,7 @@ class sRNATargetPrediction(object):
         shutil.move("tmp.txt", out_rnaup)
         return srnas
 
-    def _rnaup(self, prefixs, args_tar, log):
+    def _rnaup(self, prefixs, target_prefixs, args_tar, log):
         log.write("Using RNAup to predict sRNA targets.\n")
         log.write("Please make sure the version of Vienna RNA package is "
                   "at least 2.3.2.\n")
@@ -374,25 +439,32 @@ class sRNATargetPrediction(object):
                         if start:
                             out_up.write(line + "\n")
                             out_up.close()
-                            self.helper.merge_file(os.path.join(
-                                self.target_seq_path,
-                                "_".join([prefix, "target.fa"])),
-                                os.path.join(args_tar.out_folder,
-                                             "".join([self.tmps["tmp"],
-                                                      str(num_up), ".fa"])))
-                            if num_up == args_tar.core_up:
-                                self._run_rnaup(num_up, processes, prefix,
-                                                out_rnaup, out_log, args_tar, log)
-                                processes = []
-                                num_up = 0
-            self._run_rnaup(num_up, processes, prefix, out_rnaup, out_log,
-                            args_tar, log)
+                            for prefix in target_prefixs:
+                                self.helper.merge_file(os.path.join(
+                                    self.target_seq_path,
+                                    "_".join([prefix, "target.fa"])),
+                                    os.path.join(args_tar.out_folder,
+                                                 "".join([self.tmps["tmp"],
+                                                          str(num_up), ".fa"])))
+                                if num_up == args_tar.core_up:
+                                    self._run_rnaup(num_up, processes, prefix,
+                                                    out_rnaup, out_log, args_tar, log)
+                                    processes = []
+                                    num_up = 0
+                self._run_rnaup(num_up, processes, prefix, out_rnaup, out_log,
+                                args_tar, log)
             log.write("The prediction for {0} is done.\n".format(prefix))
             log.write("\t" + out_rnaup + " is complete generated and updated.\n")
 
-    def _intarna(self, prefixs, args_tar, log):
+    def _intarna(self, prefixs, target_prefixs, args_tar, log):
         log.write("Using IntaRNA to predict sRNA targets.\n")
         log.write("Please make sure the version of IntaRNA is at least 2.0.4.\n")
+        all_target = os.path.join(self.target_seq_path, "all_target.fa")
+        if os.path.exists(all_target):
+            os.remove(all_target)
+        for prefix in target_prefixs:
+            self.helper.merge_file(os.path.join(self.target_seq_path,
+                                   prefix + "_target.fa"), all_target)
         for prefix in prefixs:
             print("Running IntaRNA of {0}".format(prefix))
             intarna_file = os.path.join(self.intarna_path, prefix,
@@ -403,8 +475,7 @@ class sRNATargetPrediction(object):
                   "-q", os.path.join(
                       self.srna_seq_path, "_".join([
                           self.tmps["tmp"], prefix, "sRNA.fa"])),
-                  "-t", os.path.join(self.target_seq_path,
-                                     prefix + "_target.fa"),
+                  "-t", all_target,
                   "--qAccW", str(args_tar.slide_win_srna),
                   "--qAccL", str(args_tar.max_loop_srna),
                   "--tAccW", str(args_tar.slide_win_target),
@@ -415,11 +486,16 @@ class sRNATargetPrediction(object):
             log.write("The prediction for {0} is done.\n".format(prefix))
             log.write("\t" + intarna_file + " is generated.\n")
 
-    def _merge_rnaplex_rnaup(self, prefixs, args_tar, log):
+    def _merge_rnaplex_rnaup(self, prefixs, target_prefixs, args_tar, log):
         '''merge the result of IntaRNA, RNAup and RNAplex'''
         log.write("Running merge_rnaplex_rnaup.py to merge the results from "
                   "RNAplex, RNAup, and IntaRNA for generating finanl output.\n")
         log.write("The following files are generated:\n")
+        all_gff = os.path.join(self.gff_path, "all.gff")
+        if os.path.exists(all_gff):
+            os.remove(all_gff)
+        for prefix in target_prefixs:
+            self.helper.merge_file(os.path.join(self.gff_path, prefix + ".gff"), all_gff)
         for prefix in prefixs:
             rnaplex_file = None
             rnaup_file = None
@@ -436,16 +512,19 @@ class sRNATargetPrediction(object):
                 out_rnaplex = os.path.join(
                         self.rnaplex_path, prefix,
                         "_".join([prefix, "RNAplex_rank.csv"]))
+                self._remove_repeat(rnaplex_file, "RNAplex")
             if ("RNAup" in args_tar.program):
                 rnaup_file = os.path.join(self.rnaup_path, prefix,
                                           "_".join([prefix, "RNAup.txt"]))
                 out_rnaup = os.path.join(self.rnaup_path, prefix,
                                          "_".join([prefix, "RNAup_rank.csv"]))
+                self._remove_repeat(rnaup_file, "RNAup")
             if ("IntaRNA" in args_tar.program):
                 intarna_file = os.path.join(self.intarna_path, prefix,
                                             "_".join([prefix, "IntaRNA.txt"]))
                 out_intarna = os.path.join(self.intarna_path, prefix,
                                            "_".join([prefix, "IntaRNA_rank.csv"]))
+                self._remove_repeat(intarna_file, "IntaRNA")
             overlap_file = os.path.join(self.merge_path, prefix,
                                         "_".join([prefix, "overlap.csv"]))
             merge_file = os.path.join(self.merge_path, prefix,
@@ -456,7 +535,7 @@ class sRNATargetPrediction(object):
                               merge_file, overlap_file,
                               os.path.join(self.srna_path,
                                            "_".join([prefix, "sRNA.gff"])),
-                              os.path.join(self.gff_path, prefix + ".gff"))
+                              all_gff, target_prefixs)
             if ("RNAplex" in args_tar.program):
                 log.write("\t" + out_rnaplex + "\n")
             if ("RNAup" in args_tar.program):
@@ -468,6 +547,114 @@ class sRNATargetPrediction(object):
             if (os.path.exists(overlap_file)):
                 log.write("\t" + overlap_file + "\n")
 
+    def _remove_rnaplex(self, line, num, pre_num, pre, checks,
+                        out_tmp, print_):
+        if (line.startswith(">")):
+            if (num % 2 == 1):
+                print_ = False
+                pre = line
+                if (line not in checks):
+                    checks[line] = []
+                    print_ = True
+            elif (num % 2 == 0) and (line not in checks[pre]):
+                checks[pre].append(line)
+                print_ = True
+            num = num + 1
+        else:
+            if (print_):
+                if (num != pre_num):
+                    out_tmp.write(pre + "\n")
+                    out_tmp.write(checks[pre][-1] + "\n")
+                out_tmp.write(line + "\n")
+                pre_num = num
+        return num, pre_num, print_, pre,
+
+    def _remove_rnaup(self, line, pre, num, pre_num, srna_info,
+                      checks, out_tmp, print_, tar):
+        if (line.startswith(">")):
+            print_ = False
+            tar = False
+            if (pre.startswith(">")):
+                if (pre not in checks):
+                    checks[pre] = [line]
+                    srna_info = pre
+                    print_ = True
+                else:
+                    if (line not in checks[pre]):
+                        checks[pre].append(line)
+                        print_ = True
+            else:
+                if (num != 1):
+                    if (line not in checks[srna_info]):
+                        checks[srna_info].append(line)
+                        print_ = True
+        else:
+            if (print_):
+                if (pre_num != len(checks)):
+                    out_tmp.write(srna_info + "\n")
+                    out_tmp.write(checks[srna_info][-1] + "\n")
+                    out_tmp.write(line + "\n")
+                else:
+                    if (not tar):
+                        out_tmp.write(checks[srna_info][-1] + "\n")
+                    out_tmp.write(line + "\n")
+                pre_num = len(checks)
+                tar = True
+        pre = line
+        num = num + 1
+        return num, pre_num, print_, pre, tar, srna_info
+
+    def _remove_intarna(self, line, checks, tar, srna_info, seq, out_tmp):
+        if (line.startswith(".")) or (
+                line.startswith("(")) or (
+                line.startswith(")")):
+            seq = line.split(";")[0]
+            if (seq not in checks[tar][srna_info]):
+                checks[tar][srna_info].append(seq)
+                out_tmp.write(line + "\n")
+        else:
+            if (len(line.split(";")) >= 8):
+                tar = line.split(";")[0]
+                srna_info = line.split(";")[3]
+                seq = line.split(";")[7]
+                if (tar not in checks):
+                    checks[tar] = {}
+                    checks[tar][srna_info] = [seq]
+                    out_tmp.write(line + "\n")
+                else:
+                    if (srna_info not in checks[tar]):
+                        checks[tar][srna_info] = [seq]
+                        out_tmp.write(line + "\n")
+        return tar, srna_info, seq
+
+    def _remove_repeat(self, interact_file, type_):
+        checks = {}
+        seq = ""
+        pre = ""
+        srna_info = ""
+        num = 1
+        tar = False
+        pre_num = 0
+        print_ = False
+        out_tmp = open(interact_file + "tmp", "w")
+        with open(interact_file) as fh:
+            for line in fh:
+                line = line.strip()
+                if (type_ == "RNAplex"):
+                    num, pre_num, print_, pre = self._remove_rnaplex(
+                            line, num, pre_num, pre, checks, out_tmp, print_)
+                elif (type_ == "RNAup"):
+                    num, pre_num, print_, pre, tar, srna_info = (
+                            self._remove_rnaup(
+                                line, pre, num, pre_num,
+                                srna_info, checks, out_tmp, print_, tar))
+                elif (type_ == "IntaRNA"):
+                    tar, srna_info, seq = self._remove_intarna(
+                            line, checks, tar, srna_info, seq, out_tmp)
+        out_tmp.close()
+        shutil.move(interact_file + "tmp", interact_file)
+
+
     def run_srna_target_prediction(self, args_tar, log):
         self._check_gff(args_tar.gffs)
         self._check_gff(args_tar.srnas)
@@ -475,17 +662,21 @@ class sRNATargetPrediction(object):
         self.multiparser.parser_fasta(args_tar.fastas)
         self.multiparser.parser_gff(args_tar.srnas, "sRNA")
         prefixs = []
-        self._gen_seq(prefixs, args_tar)
+        target_prefixs = []
+        self._gen_seq(prefixs, target_prefixs, args_tar)
         if ("RNAplex" in args_tar.program):
-            self._rna_plex(prefixs, args_tar, log)
+            self._rna_plex(prefixs, target_prefixs, args_tar, log)
         self.helper.remove_all_content(self.target_seq_path,
                                        "_target_", "file")
+        if os.path.exists(os.path.join(self.rnaplex_path, "tmp_RNAplfold")):
+            shutil.rmtree(os.path.join(self.rnaplex_path,
+                                       "tmp_RNAplfold"))
         log.write("The temporary files for running RNAplex are deleted.\n")
         if ("RNAup" in args_tar.program):
-            self._rnaup(prefixs, args_tar, log)
+            self._rnaup(prefixs, target_prefixs, args_tar, log)
         if ("IntaRNA" in args_tar.program):
-            self._intarna(prefixs, args_tar, log)
-        self._merge_rnaplex_rnaup(prefixs, args_tar, log)
+            self._intarna(prefixs, target_prefixs, args_tar, log)
+        self._merge_rnaplex_rnaup(prefixs, target_prefixs, args_tar, log)
         self.helper.remove_all_content(args_tar.out_folder,
                                        self.tmps["tmp"], "dir")
         self.helper.remove_all_content(args_tar.out_folder,
@@ -494,3 +685,5 @@ class sRNATargetPrediction(object):
         self.helper.remove_tmp_dir(args_tar.srnas)
         self.helper.remove_tmp_dir(args_tar.fastas)
         self.helper.remove_all_content(self.srna_seq_path, "tmp_", "file")
+        if os.path.exists(os.path.join(self.target_seq_path, "all_target.fa")):
+            os.remove(os.path.join(self.target_seq_path, "all_target.fa"))
